# Generated by Django 3.1 on 2020-08-21 19:49

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('helpme', '0002_auto_20200814_1841'),
    ]

    operations = [
        migrations.AlterModelOptions(
            name='ticket',
            options={'permissions': (('see-support-tickets', 'Access to support-level tickets'), ('see-developer-tickets', 'Access to developer-level tickets'), ('see-all-tickets', 'Access to all tickets'))},
        ),
        migrations.AlterField(
            model_name='ticket',
            name='history',
            field=models.JSONField(blank=True, default=list, null=True),
        ),
        migrations.AlterField(
            model_name='ticket',
            name='priority',
            field=models.IntegerField(choices=[(1, 'Suggestion'), (2, 'Low'), (3, 'Medium'), (4, 'High'), (5, 'Urgent')], default=3, verbose_name='Priority'),
        ),
        migrations.AlterField(
            model_name='ticket',
            name='user_meta',
            field=models.JSONField(blank=True, default=dict, null=True),
        ),
    ]



# Generated by Django 3.1 on 2020-08-24 19:50

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('sites', '0002_alter_domain_unique'),
        ('helpme', '0006_ticket_site'),
    ]

    operations = [
        migrations.RemoveField(
            model_name='team',
            name='site',
        ),
        migrations.AddField(
            model_name='team',
            name='sites',
            field=models.ManyToManyField(default=1, to='sites.Site'),
        ),
    ]



# Generated by Django 3.1 on 2020-08-24 17:50

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        ('sites', '0002_alter_domain_unique'),
        ('helpme', '0005_auto_20200821_2224'),
    ]

    operations = [
        migrations.AddField(
            model_name='ticket',
            name='site',
            field=models.ForeignKey(default=1, on_delete=django.db.models.deletion.CASCADE, to='sites.site'),
        ),
    ]



# Generated by Django 3.1 on 2020-08-21 21:55

from django.db import migrations, models
import django.db.models.deletion
import multiselectfield.db.fields


class Migration(migrations.Migration):

    dependencies = [
        ('sites', '0002_alter_domain_unique'),
        ('helpme', '0003_auto_20200821_1949'),
    ]

    operations = [
        migrations.AddField(
            model_name='team',
            name='categories',
            field=multiselectfield.db.fields.MultiSelectField(choices=[(1, 'Comment'), (2, 'Sales'), (3, 'Help'), (4, 'Bug')], default=3, max_length=7),
        ),
        migrations.AddField(
            model_name='team',
            name='site',
            field=models.ForeignKey(default=1, on_delete=django.db.models.deletion.CASCADE, to='sites.site'),
        ),
    ]



# Generated by Django 3.0 on 2020-08-14 18:41

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('helpme', '0001_initial'),
    ]

    operations = [
        migrations.AlterField(
            model_name='ticket',
            name='category',
            field=models.IntegerField(choices=[(1, 'Comment'), (2, 'Sales'), (3, 'Help'), (4, 'Bug')], default=3, verbose_name='Category'),
        ),
        migrations.AlterField(
            model_name='ticket',
            name='priority',
            field=models.IntegerField(choices=[(1, 'Low'), (2, 'Medium'), (3, 'High'), (4, 'Urgent')], default=2, verbose_name='Priority'),
        ),
    ]



# Generated by Django 3.1 on 2020-08-21 22:24

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('helpme', '0004_auto_20200821_2155'),
    ]

    operations = [
        migrations.RemoveField(
            model_name='ticket',
            name='team',
        ),
        migrations.AddField(
            model_name='ticket',
            name='teams',
            field=models.ManyToManyField(blank=True, related_name='support_tickets', to='helpme.Team'),
        ),
    ]




from django.views.generic import TemplateView
from django.views.generic.edit import CreateView, UpdateView
from django.views.generic.list import ListView
from django.urls import reverse_lazy
from django.conf import settings
from django.http import JsonResponse
from django.contrib.auth.mixins import LoginRequiredMixin, PermissionRequiredMixin
from django.contrib.sites.models import Site

from .models import Ticket, Team
from .forms import CommentForm


class SupportRequestView(LoginRequiredMixin, CreateView):
    model = Ticket
    success_url = reverse_lazy('helpme:success')
    category = 3
    fields = ['subject', 'description', 'category']

    def form_invalid(self, form):
        response = super().form_invalid(form)
        if self.request.is_ajax():
            return JsonResponse(form.errors, status=400)
        else:
            return response

    def form_valid(self, form):
        form.instance.category = self.category
        form.instance.user = self.request.user
        form.instance.site = Site.objects.get_current()
        
        form.instance.log_history_event(event="created", user=self.request.user)

        response = super().form_valid(form)

        # filter and assign teams by site and category
        teams = Team.objects.filter(sites__in=[form.instance.site])
        form.instance.teams.set(teams.filter(categories__contains=self.category))
        form.instance.save()

        if self.request.is_ajax():
            data = {
                'message': "Successfully submitted form data."
            }
            return JsonResponse(data)
        else:
            return response

            
class SupportRequestSuccessView(LoginRequiredMixin, TemplateView):
    template_name = "helpme/submission_successful.html"


class SupportDashboardView(LoginRequiredMixin, ListView):
    model = Ticket

    def get_queryset(self, **kwargs):
        # supervisor
        if self.request.user.has_perm('helpme.see-all-tickets'):
            queryset = Ticket.objects.all()
        # support team member
        # sees tickets that are assigned to them or to a team they belong to
        # but are not assigned to a specific user yet
        elif self.request.user.has_perm('helpme.see-support-tickets'):
            tickets = Ticket.objects.none()
            for team in self.request.user.team_set.all():
                tickets = tickets | Ticket.objects.filter(teams__in=[team])
            queryset = Ticket.objects.filter(assigned_to=self.request.user) | tickets.filter(assigned_to=None)
        # platform user
        else:
            queryset = Ticket.objects.filter(user=self.request.user)
        return queryset.order_by('-priority')

    def get_context_data(self, **kwargs):
        context = super().get_context_data(**kwargs)
        context['support'] = self.request.user.has_perm('see-support-tickets')
        return context
    

class TicketDetailView(LoginRequiredMixin, UpdateView):
    model = Ticket
    fields = ['status', 'priority', 'category', 'teams', 'assigned_to', 'dev_ticket', 'related_to']
    template_name = "helpme/ticket_detail.html"

    def get_success_url(self):
        return reverse_lazy('helpme:ticket-detail', args=[self.object.uuid])
    
    def get_object(self, queryset=None):
        if queryset is None:
            queryset = self.get_queryset()

        uuid = self.kwargs.get('uuid')

        try:
            obj = queryset.get(uuid=uuid)
        except queryset.model.DoesNotExist:
            raise Http404(_("No %(verbose_name)s found matching the query") %
                          {'verbose_name': queryset.model._meta.verbose_name})
        return obj

    def get_context_data(self, **kwargs):
        context = super().get_context_data(**kwargs)
        support = self.request.user.has_perm('helpme.see-support-tickets')
        
        context['support'] = support
        context['developer'] = self.request.user.has_perm('helpme.see-developer-tickets')
        context['supervisor'] = self.request.user.has_perm('helpme.see-all-tickets')

        context['user'] = self.request.user
        context['comment_form'] = CommentForm(support=support)
        context['comments'] = self.object.comments.all()
        return context

    def form_valid(self, form):
        form.instance.log_history_event(event="updated", user=self.request.user)
        response = super().form_valid(form)
        return response


class TeamCreateView(LoginRequiredMixin, PermissionRequiredMixin, CreateView):
    permission_required = 'helpme.view_team'
    model = Team
    fields = ['name', 'global_team', 'sites', 'categories', 'members']
    success_url = reverse_lazy('helpme:team-list')

    def get_context_data(self, **kwargs):
        context = super().get_context_data(**kwargs)
        context['teams'] = Team.objects.all()
        context['admin'] = self.request.user.has_perm('helpme.add_team')
        context['user_teams'] = self.request.user.team_set.all()
        return context

    def form_valid(self, form):
        response = super().form_valid(form)
        if form.instance.global_team:
            form.instance.sites.set(Site.objects.all())
            form.instance.save()
        return response
    
class TeamDetailView(LoginRequiredMixin, PermissionRequiredMixin, UpdateView):
    permission_required = 'helpme.view_team'
    model = Team
    fields = ['name', 'global_team', 'sites', 'categories', 'members']
    template_name = "helpme/team_detail.html"

    def get_success_url(self):
        return reverse_lazy('helpme:team-detail', args=[self.object.uuid])

    def get_object(self, queryset=None):
        if queryset is None:
            queryset = self.get_queryset()

        uuid = self.kwargs.get('uuid')

        try:
            obj = queryset.get(uuid=uuid)
        except queryset.model.DoesNotExist:
            raise Http404(_("No %(verbose_name)s found matching the query") %
                          {'verbose_name': queryset.model._meta.verbose_name})
        return obj

    def form_valid(self, form):
        response = super().form_valid(form)
        if form.instance.global_team:
            form.instance.sites.set(Site.objects.all())
            form.instance.save()
        return response

    def get_context_data(self, **kwargs):
        context = super().get_context_data(**kwargs)
        context['admin'] = self.request.user.has_perm('helpme.change_team')
        return context



from django.forms import ModelForm
from django import forms
from django.utils.translation import gettext as _

from helpme.models import Ticket, Comment


class SupportRequestForm(ModelForm):

    class Meta:
        model = Ticket
        fields = ['subject', 'description']


class CommentForm(ModelForm):
    content = forms.CharField(label="", widget=forms.Textarea(attrs={'placeholder': 'Leave a comment'}))
    
    class Meta:
        model = Comment
        fields = ['content', 'visibility']

    def __init__(self, support=True, *args, **kwargs):
        super().__init__(*args, **kwargs)
        if not support:
            self.fields.pop('visibility')
            
